@@ -8,22 +8,14 @@ module Sequel
       module DatabaseMethods
         include Sequel::SQLite::DatabaseMethods
         
-        # Return Sequel::JDBC::SQLite::Dataset object with the given opts.
-        def dataset(opts=nil)
-          Sequel::JDBC::SQLite::Dataset.new(self, opts)
-        end
-        
         private
         
         # Use last_insert_rowid() to get the last inserted id.
         def last_insert_id(conn, opts={})
-          stmt = conn.createStatement
-          begin
+          statement(conn) do |stmt|
             rs = stmt.executeQuery('SELECT last_insert_rowid()')
             rs.next
             rs.getInt(1)
-          ensure
-            stmt.close
           end
         end
         
@@ -32,6 +24,15 @@ module Sequel
           o = super
           uri == 'jdbc:sqlite::memory:' ? o.merge(:max_connections=>1) : o
         end
+        
+        # Execute the connection pragmas on the connection.
+        def setup_connection(conn)
+          conn = super(conn)
+          statement(conn) do |stmt|
+            connection_pragmas.each{|s| log_yield(s){stmt.execute(s)}}
+          end
+          conn
+        end
       end
       
       # Dataset class for SQLite datasets accessed via JDBC.
