@@ -6,12 +6,6 @@ module Sequel
       def database_type
         :progress
       end
-
-      def dataset(opts = nil)
-        ds = super
-        ds.extend(DatasetMethods)
-        ds
-      end
     end
   
     module DatasetMethods
