@@ -8,10 +8,12 @@ module Sequel
       # Database instance methods for SQLite databases accessed via Swift.
       module DatabaseMethods
         include Sequel::SQLite::DatabaseMethods
-        
-        # Return instance of Sequel::Swift::SQL::Dataset with the given opts.
-        def dataset(opts=nil)
-          Sequel::Swift::SQLite::Dataset.new(self, opts)
+
+        # Set the correct pragmas on the connection.
+        def connect(opts)
+          c = super
+          connection_pragmas.each{|s| log_yield(s){c.execute(s)}}
+          c
         end
       end
       
