@@ -1,11 +1,23 @@
 module Sequel
   class Dataset
+    # ---------------------
+    # :section: 5 - Methods related to dataset graphing
+    # Dataset graphing changes the dataset to yield hashes where keys are table
+    # name symbols and values are hashes representing the columns related to
+    # that table.  All of these methods return modified copies of the receiver.
+    # ---------------------
+    
     # Adds the given graph aliases to the list of graph aliases to use,
-    # unlike #set_graph_aliases, which replaces the list.  See
-    # #set_graph_aliases.
+    # unlike +set_graph_aliases+, which replaces the list (the equivalent
+    # of +select_more+ when graphing).  See +set_graph_aliases+.
+    #
+    #   DB[:table].add_graph_aliases(:some_alias=>[:table, :column])
+    #   # SELECT ..., table.column AS some_alias
+    #   # => {:table=>{:column=>some_alias_value, ...}, ...}
     def add_graph_aliases(graph_aliases)
-      ds = select_more(*graph_alias_columns(graph_aliases))
-      ds.opts[:graph_aliases] = (ds.opts[:graph_aliases] || ds.opts[:graph][:column_aliases] || {}).merge(graph_aliases)
+      columns, graph_aliases = graph_alias_columns(graph_aliases)
+      ds = select_more(*columns)
+      ds.opts[:graph_aliases] = (ds.opts[:graph_aliases] || (ds.opts[:graph][:column_aliases] rescue {}) || {}).merge(graph_aliases)
       ds
     end
 
@@ -17,16 +29,18 @@ module Sequel
     #
     #   # CREATE TABLE artists (id INTEGER, name TEXT);
     #   # CREATE TABLE albums (id INTEGER, name TEXT, artist_id INTEGER);
+    #
     #   DB[:artists].left_outer_join(:albums, :artist_id=>:id).first
-    #   => {:id=>albums.id, :name=>albums.name, :artist_id=>albums.artist_id}
+    #   #=> {:id=>albums.id, :name=>albums.name, :artist_id=>albums.artist_id}
+    #
     #   DB[:artists].graph(:albums, :artist_id=>:id).first
-    #   => {:artists=>{:id=>artists.id, :name=>artists.name}, :albums=>{:id=>albums.id, :name=>albums.name, :artist_id=>albums.artist_id}}
+    #   #=> {:artists=>{:id=>artists.id, :name=>artists.name}, :albums=>{:id=>albums.id, :name=>albums.name, :artist_id=>albums.artist_id}}
     #
     # Using a join such as left_outer_join, the attribute names that are shared between
     # the tables are combined in the single return hash.  You can get around that by
-    # using .select with correct aliases for all of the columns, but it is simpler to
-    # use graph and have the result set split for you.  In addition, graph respects
-    # any row_proc of the current dataset and the datasets you use with graph.
+    # using +select+ with correct aliases for all of the columns, but it is simpler to
+    # use +graph+ and have the result set split for you.  In addition, +graph+ respects
+    # any +row_proc+ of the current dataset and the datasets you use with +graph+.
     #
     # If you are graphing a table and all columns for that table are nil, this
     # indicates that no matching rows existed in the table, so graph will return nil
@@ -37,26 +51,26 @@ module Sequel
     #   => {:artists=>{:id=>artists.id, :name=>artists.name}, :albums=>nil}
     #
     # Arguments:
-    # * dataset -  Can be a symbol (specifying a table), another dataset,
-    #   or an object that responds to .dataset and return a symbol or a dataset
-    # * join_conditions - Any condition(s) allowed by join_table.
-    # * options -  A hash of graph options.  The following options are currently used:
-    #   * :from_self_alias - The alias to use when the receiver is not a graphed
-    #     dataset but it contains multiple FROM tables or a JOIN.  In this case,
-    #     the receiver is wrapped in a from_self before graphing, and this option
-    #     determines the alias to use.
-    #   * :implicit_qualifier - The qualifier of implicit conditions, see #join_table.
-    #   * :join_type - The type of join to use (passed to join_table).  Defaults to
-    #     :left_outer.
-    #   * :select - An array of columns to select.  When not used, selects
-    #     all columns in the given dataset.  When set to false, selects no
-    #     columns and is like simply joining the tables, though graph keeps
-    #     some metadata about join that makes it important to use graph instead
-    #     of join.
-    #   * :table_alias - The alias to use for the table.  If not specified, doesn't
-    #     alias the table.  You will get an error if the the alias (or table) name is
-    #     used more than once.
-    # * block - A block that is passed to join_table.
+    # dataset :: Can be a symbol (specifying a table), another dataset,
+    #            or an object that responds to +dataset+ and returns a symbol or a dataset
+    # join_conditions :: Any condition(s) allowed by +join_table+.
+    # block :: A block that is passed to +join_table+.
+    #
+    # Options:
+    # :from_self_alias :: The alias to use when the receiver is not a graphed
+    #                     dataset but it contains multiple FROM tables or a JOIN.  In this case,
+    #                     the receiver is wrapped in a from_self before graphing, and this option
+    #                     determines the alias to use.
+    # :implicit_qualifier :: The qualifier of implicit conditions, see #join_table.
+    # :join_type :: The type of join to use (passed to +join_table+).  Defaults to :left_outer.
+    # :select :: An array of columns to select.  When not used, selects
+    #            all columns in the given dataset.  When set to false, selects no
+    #            columns and is like simply joining the tables, though graph keeps
+    #            some metadata about the join that makes it important to use +graph+ instead
+    #            of +join_table+.
+    # :table_alias :: The alias to use for the table.  If not specified, doesn't
+    #                 alias the table.  You will get an error if the the alias (or table) name is
+    #                 used more than once.
     def graph(dataset, join_conditions = nil, options = {}, &block)
       # Allow the use of a model, dataset, or symbol as the first argument
       # Find the table name/dataset based on the argument
@@ -99,10 +113,12 @@ module Sequel
       add_table = options[:select] == false ? false : true
       # Whether to add the columns to the list of column aliases
       add_columns = !ds.opts.include?(:graph_aliases)
+      # columns to select
+      select = (opts[:select] || []).dup
 
       # Setup the initial graph data structure if it doesn't exist
       unless graph = opts[:graph]
-        master = ds.first_source_alias
+        master = alias_symbol(ds.first_source_alias)
         raise_alias_error.call if master == table_alias
         # Master hash storing all .graph related information
         graph = opts[:graph] = {}
@@ -116,7 +132,6 @@ module Sequel
         # aliased, but are not included if set_graph_aliases
         # has been used.
         if add_columns
-          select = opts[:select] = []
           columns.each do |column|
             column_aliases[column] = [master, column]
             select.push(SQL::QualifiedIdentifier.new(master, column))
@@ -133,7 +148,6 @@ module Sequel
 
       # Add the columns to the selection unless we are ignoring them
       if add_table && add_columns
-        select = opts[:select]
         column_aliases = graph[:column_aliases]
         ca_num = graph[:column_alias_num]
         # Which columns to add to the result set
@@ -151,56 +165,71 @@ module Sequel
               column_alias = :"#{column_alias}_#{column_alias_num}" 
               ca_num[column_alias] += 1
             end
-            [column_alias, SQL::QualifiedIdentifier.new(table_alias, column).as(column_alias)]
+            [column_alias, SQL::AliasedExpression.new(SQL::QualifiedIdentifier.new(table_alias, column), column_alias)]
           else
-            [column, SQL::QualifiedIdentifier.new(table_alias, column)]
+            ident = SQL::QualifiedIdentifier.new(table_alias, column)
+            [column, ident]
           end
           column_aliases[col_alias] = [table_alias, column]
           select.push(identifier)
         end
       end
-      ds
+      ds.select(*select)
     end
 
     # This allows you to manually specify the graph aliases to use
     # when using graph.  You can use it to only select certain
     # columns, and have those columns mapped to specific aliases
-    # in the result set.  This is the equivalent of .select for a
-    # graphed dataset, and must be used instead of .select whenever
-    # graphing is used. Example:
+    # in the result set.  This is the equivalent of +select+ for a
+    # graphed dataset, and must be used instead of +select+ whenever
+    # graphing is used.
     #
-    #   DB[:artists].graph(:albums, :artist_id=>:id).set_graph_aliases(:artist_name=>[:artists, :name], :album_name=>[:albums, :name], :forty_two=>[:albums, :fourtwo, 42]).first
-    #   => {:artists=>{:name=>artists.name}, :albums=>{:name=>albums.name, :fourtwo=>42}}
+    # graph_aliases :: Should be a hash with keys being symbols of
+    #                  column aliases, and values being either symbols or arrays with one to three elements.
+    #                  If the value is a symbol, it is assumed to be the same as a one element
+    #                  array containing that symbol.
+    #                  The first element of the array should be the table alias symbol.
+    #                  The second should be the actual column name symbol.  If the array only
+    #                  has a single element the column name symbol will be assumed to be the
+    #                  same as the corresponding hash key. If the array
+    #                  has a third element, it is used as the value returned, instead of
+    #                  table_alias.column_name.
     #
-    # Arguments:
-    # * graph_aliases - Should be a hash with keys being symbols of
-    #   column aliases, and values being arrays with two or three elements.
-    #   The first element of the array should be the table alias symbol,
-    #   and the second should be the actual column name symbol. If the array
-    #   has a third element, it is used as the value returned, instead of
-    #   table_alias.column_name.
+    #   DB[:artists].graph(:albums, :artist_id=>:id).
+    #     set_graph_aliases(:name=>:artists,
+    #                       :album_name=>[:albums, :name],
+    #                       :forty_two=>[:albums, :fourtwo, 42]).first
+    #   # SELECT artists.name, albums.name AS album_name, 42 AS forty_two ...
+    #   # => {:artists=>{:name=>artists.name}, :albums=>{:name=>albums.name, :fourtwo=>42}}
     def set_graph_aliases(graph_aliases)
-      ds = select(*graph_alias_columns(graph_aliases))
+      columns, graph_aliases = graph_alias_columns(graph_aliases)
+      ds = select(*columns)
       ds.opts[:graph_aliases] = graph_aliases
       ds
     end
 
-    # Remove the splitting of results into subhashes.  Also removes
-    # metadata related to graphing, so you should not call graph
-    # any tables to this dataset after calling this method.
+    # Remove the splitting of results into subhashes, and all metadata
+    # related to the current graph (if any).
     def ungraphed
-      clone(:graph=>nil)
+      clone(:graph=>nil, :graph_aliases=>nil)
     end
 
     private
 
-    # Transform the hash of graph aliases to an array of columns
+    # Transform the hash of graph aliases and return a two element array
+    # where the first element is an array of identifiers suitable to pass to
+    # a select method, and the second is a new hash of preprocessed graph aliases.
     def graph_alias_columns(graph_aliases)
-      graph_aliases.collect do |col_alias, tc| 
-        identifier = tc[2] || SQL::QualifiedIdentifier.new(tc[0], tc[1])
-        identifier = SQL::AliasedExpression.new(identifier, col_alias) if tc[2] or tc[1] != col_alias
+      gas = {}
+      identifiers = graph_aliases.collect do |col_alias, tc| 
+        table, column, value = Array(tc)
+        column ||= col_alias
+        gas[col_alias] = [table, column]
+        identifier = value || SQL::QualifiedIdentifier.new(table, column)
+        identifier = SQL::AliasedExpression.new(identifier, col_alias) if value || column != col_alias
         identifier
       end
+      [identifiers, gas]
     end
 
     # Fetch the rows, split them into component table parts,
