@@ -1,5 +1,8 @@
-require File.join(File.dirname(__FILE__), 'spec_helper')
+require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
 
+if defined?(ActiveSupport::Inflector)
+  skip_warn "inflector extension: active_support inflector loaded"
+else
 describe String do
   it "#camelize and #camelcase should transform the word to CamelCase" do
     "egg_and_hams".camelize.should == "EggAndHams"
@@ -159,7 +162,6 @@ describe 'Default inflections' do
       :index=>:indices,
       :mouse=>:mice,
       :louse=>:lice,
-      :ox=>:oxen,
       :quiz=>:quizzes,
       :motive=>:motives,
       :movie=>:movies,
@@ -180,3 +182,4 @@ describe 'Default inflections' do
     end
   end
 end
+end
