@@ -1,8 +1,11 @@
-require File.join(File.dirname(__FILE__), "spec_helper")
+require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
+begin
 require 'yaml'
 require 'json'
-
+rescue LoadError => e
+  skip_warn "serialization plugin: can't load json (#{e.class}: #{e})"
+else
 describe "Serialization plugin" do
   before do
     @c = Class.new(Sequel::Model(:items)) do
@@ -19,11 +22,11 @@ describe "Serialization plugin" do
   it "should allow setting additional serializable attributes via plugin :serialization call" do
     @c.plugin :serialization, :yaml, :abc
     @c.create(:abc => 1, :def=> 2)
-    MODEL_DB.sqls.last.should =~ /INSERT INTO items \((abc, def|def, abc)\) VALUES \(('--- 1\n', 2|2, '--- 1\n')\)/
+    MODEL_DB.sqls.last.should =~ /INSERT INTO items \((abc, def|def, abc)\) VALUES \(('--- 1\n(\.\.\.\n)?', 2|2, '--- 1\n(\.\.\.\n)?')\)/
 
     @c.plugin :serialization, :marshal, :def
     @c.create(:abc => 1, :def=> 1)
-    MODEL_DB.sqls.last.should =~ /INSERT INTO items \((abc, def|def, abc)\) VALUES \(('--- 1\n', 'BAhpBg==\n'|'BAhpBg==\n', '--- 1\n')\)/
+    MODEL_DB.sqls.last.should =~ /INSERT INTO items \((abc, def|def, abc)\) VALUES \(('--- 1\n(\.\.\.\n)?', 'BAhpBg==\n'|'BAhpBg==\n', '--- 1\n(\.\.\.\n)?')\)/
     
     @c.plugin :serialization, :json, :ghi
     @c.create(:ghi => [123])
@@ -35,10 +38,7 @@ describe "Serialization plugin" do
     @c.create(:abc => 1)
     @c.create(:abc => "hello")
 
-    MODEL_DB.sqls.should == [ \
-      "INSERT INTO items (abc) VALUES ('--- 1\n')", \
-      "INSERT INTO items (abc) VALUES ('--- hello\n')", \
-    ]
+    MODEL_DB.sqls.map{|s| s.sub("...\n", '')}.should == ["INSERT INTO items (abc) VALUES ('--- 1\n')", "INSERT INTO items (abc) VALUES ('--- hello\n')"]
   end
 
   it "serialization_format should be the serialization format used" do
@@ -68,7 +68,7 @@ describe "Serialization plugin" do
     @c.create(:ghi => [1])
     @c.create(:ghi => ["hello"])
     
-    x = JSON.generate ["hello"]
+    x = ["hello"].to_json
     MODEL_DB.sqls.should == [ \
       "INSERT INTO items (ghi) VALUES ('[1]')", \
       "INSERT INTO items (ghi) VALUES ('#{x}')", \
@@ -94,7 +94,7 @@ describe "Serialization plugin" do
 
     o.update(:abc => 23)
     @c.create(:abc => [1, 2, 3])
-    MODEL_DB.sqls.should == ["UPDATE items SET abc = '--- 23\n' WHERE (id = 1)",
+    MODEL_DB.sqls.should == ["UPDATE items SET abc = '#{23.to_yaml}' WHERE (id = 1)",
       "INSERT INTO items (abc) VALUES ('#{[1, 2, 3].to_yaml}')"]
   end
 
@@ -126,7 +126,7 @@ describe "Serialization plugin" do
     
     ds = @c.dataset
     def ds.fetch_rows(sql, &block)
-      block.call(:id => 1, :abc => JSON.generate([1]), :def => JSON.generate(["hello"]))
+      block.call(:id => 1, :abc => [1].to_json, :def => ["hello"].to_json)
     end
     
     o = @c.first
@@ -139,8 +139,8 @@ describe "Serialization plugin" do
     o.update(:abc => [23])
     @c.create(:abc => [1,2,3])
     
-    MODEL_DB.sqls.should == ["UPDATE items SET abc = '#{JSON.generate([23])}' WHERE (id = 1)",
-      "INSERT INTO items (abc) VALUES ('#{JSON.generate([1,2,3])}')"]
+    MODEL_DB.sqls.should == ["UPDATE items SET abc = '#{[23].to_json}' WHERE (id = 1)",
+      "INSERT INTO items (abc) VALUES ('#{[1,2,3].to_json}')"]
   end
 
   it "should copy serialization formats and columns to subclasses" do
@@ -161,7 +161,7 @@ describe "Serialization plugin" do
 
     o.update(:abc => 23)
     Class.new(@c).create(:abc => [1, 2, 3])
-    MODEL_DB.sqls.should == ["UPDATE items SET abc = '--- 23\n' WHERE (id = 1)",
+    MODEL_DB.sqls.should == ["UPDATE items SET abc = '#{23.to_yaml}' WHERE (id = 1)",
       "INSERT INTO items (abc) VALUES ('#{[1, 2, 3].to_yaml}')"]
   end
 
@@ -207,3 +207,4 @@ describe "Serialization plugin" do
     o.abc.should == 9
   end
 end
+end
