@@ -9,14 +9,54 @@ begin
 rescue LoadError
 end
 
+if ENV['SEQUEL_COLUMNS_INTROSPECTION']
+  Sequel.extension :columns_introspection
+  Sequel::Dataset.introspect_all_columns
+end
+
 Sequel::Model.use_transactions = false
 
-$sqls = []
-def clear_sqls
-  $sqls.clear
-end 
+unless defined?(RSpec)
+  module Spec::Matchers
+    class BeWithin
+      include Spec::Matchers
+      def initialize(delta); @delta = delta; end
+      def of(expected); be_close(expected, @delta); end 
+    end
+    def be_within(delta)
+      BeWithin.new(delta)
+    end
+  end
+end
 
-class Spec::Example::ExampleGroup
+def Sequel.guarded?(*checked)
+  unless ENV['SEQUEL_NO_PENDING']
+    checked.each do |c|
+      case c
+      when INTEGRATION_DB.database_type
+        return c
+      when Array
+        case c.length
+        when 1
+          return c if c.first == INTEGRATION_DB.adapter_scheme
+        when 2
+          if c.first.is_a?(Proc)
+            return c if c.first.call(INTEGRATION_DB) && c.last == INTEGRATION_DB.database_type
+          elsif c.last.is_a?(Proc)
+            return c if c.first == INTEGRATION_DB.adapter_scheme && c.last.call(INTEGRATION_DB)
+          else
+            return c if c.first == INTEGRATION_DB.adapter_scheme && c.last == INTEGRATION_DB.database_type
+          end
+        when 3
+          return c if c[0] == INTEGRATION_DB.adapter_scheme && c[1] == INTEGRATION_DB.database_type && c[2].call(INTEGRATION_DB)
+        end          
+      end
+    end
+  end
+  false
+end
+
+(defined?(RSpec) ? RSpec::Core::ExampleGroup : Spec::Example::ExampleGroup).class_eval do
   def log
     begin
       INTEGRATION_DB.loggers << Logger.new(STDOUT)
@@ -26,35 +66,8 @@ class Spec::Example::ExampleGroup
     end
   end
   
-  def self.log_specify(message, &block)
-    specify(message){log{instance_eval(&block)}}
-  end
-
   def self.cspecify(message, *checked, &block)
-    pending = false
-    checked.each do |c|
-      case c
-      when INTEGRATION_DB.database_type
-        pending = c
-      when Array
-        case c.length
-        when 1
-          pending = c if c.first == INTEGRATION_DB.adapter_scheme
-        when 2
-          if c.first.is_a?(Proc)
-            pending = c if c.first.call(INTEGRATION_DB) && c.last == INTEGRATION_DB.database_type
-          elsif c.last.is_a?(Proc)
-            pending = c if c.first == INTEGRATION_DB.adapter_scheme && c.last.call(INTEGRATION_DB)
-          else
-            pending = c if c.first == INTEGRATION_DB.adapter_scheme && c.last == INTEGRATION_DB.database_type
-          end
-        when 3
-          pending = c if c[0] == INTEGRATION_DB.adapter_scheme && c[1] == INTEGRATION_DB.database_type && c[2].call(INTEGRATION_DB)
-        end          
-      end
-      break if pending
-    end
-    if pending
+    if pending = Sequel.guarded?(*checked)
       specify(message){pending("Not yet working on #{Array(pending).join(', ')}", &block)}
     else
       specify(message, &block)
@@ -69,5 +82,14 @@ if defined?(INTEGRATION_DB) || defined?(INTEGRATION_URL) || ENV['SEQUEL_INTEGRAT
     #INTEGRATION_DB.instance_variable_set(:@server_version, 80100)
   end
 else
-  INTEGRATION_DB = Sequel.sqlite('', :quote_identifiers=>false)
+  INTEGRATION_DB = Sequel.sqlite
+end
+
+if INTEGRATION_DB.adapter_scheme == :ibmdb
+  def INTEGRATION_DB.drop_table(*tables)
+    super
+  rescue Sequel::DatabaseError
+    disconnect
+    super
+  end
 end
