@@ -1,4 +1,4 @@
-require File.join(File.dirname(__FILE__), "spec_helper")
+require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
 describe Sequel::Model::Errors do
   before do
@@ -12,22 +12,34 @@ describe Sequel::Model::Errors do
     @errors.should == {}
   end
   
-  specify "should be empty if no errors are added" do
+  specify "should be empty if there are no errors" do
     @errors.should be_empty
+    @errors[:blah]
+    @errors.should be_empty
+  end
+  
+  specify "should not be empty if there are errors" do
     @errors[:blah] << "blah"
     @errors.should_not be_empty
   end
   
-  specify "should return errors for a specific attribute using #on or #[]" do
+  specify "should return errors for a specific attribute using #[]" do
     @errors[:blah].should == []
-    @errors.on(:blah).should == []
-
     @errors[:blah] << 'blah'
     @errors[:blah].should == ['blah']
-    @errors.on(:blah).should == ['blah']
 
     @errors[:bleu].should == []
-    @errors.on(:bleu).should == []
+  end
+  
+  specify "should return an array of errors for a specific attribute using #on if there are errors" do
+    @errors[:blah] << 'blah'
+    @errors.on(:blah).should == ['blah']
+  end
+  
+  specify "should return nil using #on if there are no errors for that attribute" do
+    @errors.on(:blah).should == nil
+    @errors[:blah]
+    @errors.on(:blah).should == nil
   end
   
   specify "should accept errors using #[] << or #add" do
@@ -49,6 +61,17 @@ describe Sequel::Model::Errors do
     msgs.should include('blow blieuh', 'blow blich', 'blay bliu')
   end
 
+  specify "should not add column names for LiteralStrings" do
+    @errors.full_messages.should == []
+    
+    @errors[:blow] << 'blieuh'
+    @errors[:blow] << 'blich'.lit
+    @errors[:blay] << 'bliu'
+    msgs = @errors.full_messages
+    msgs.size.should == 3
+    msgs.should include('blow blieuh', 'blich', 'blay bliu')
+  end
+
   specify "should return the number of error messages using #count" do
     @errors.count.should == 0
     @errors.add(:a, 'b')
@@ -104,6 +127,10 @@ describe Sequel::Model do
     @o.errors[:score].should == ['too low']
     @o.errors[:blah].should be_empty
   end
+  
+  specify "should allow raising of ValidationFailed with a string" do
+    proc{raise Sequel::ValidationFailed, "no reason"}.should raise_error(Sequel::ValidationFailed, "no reason")
+  end
 end
 
 describe "Model#save" do
@@ -112,7 +139,7 @@ describe "Model#save" do
       columns :id, :x
 
       def validate
-        errors[:id] << 'blah' unless id == 5
+        errors.add(:id, 'blah') unless x == 7
       end
     end
     @m = @c.load(:id => 4, :x=>6)
@@ -125,10 +152,10 @@ describe "Model#save" do
     @m.save
     MODEL_DB.sqls.should be_empty
     
-    @m.id = 5
+    @m.x = 7
     @m.should be_valid
     @m.save.should_not be_false
-    MODEL_DB.sqls.should == ['UPDATE people SET x = 6 WHERE (id = 5)']
+    MODEL_DB.sqls.should == ['UPDATE people SET x = 7 WHERE (id = 4)']
   end
   
   specify "should skip validations if the :validate=>false option is used" do
@@ -137,11 +164,16 @@ describe "Model#save" do
     @m.save(:validate=>false)
     MODEL_DB.sqls.should == ['UPDATE people SET x = 6 WHERE (id = 4)']
   end
-    
+
   specify "should raise error if validations fail and raise_on_save_faiure is true" do
-    proc{@m.save}.should raise_error(Sequel::ValidationFailed)
+    proc{@m.save}.should raise_error(Sequel::ValidationFailed){|e| e.errors.should == @m.errors }
   end
-  
+
+  specify "should raise error if validations fail and :raise_on_failure option is true" do
+    @m.raise_on_save_failure = false
+    proc{@m.save(:raise_on_failure => true)}.should raise_error(Sequel::ValidationFailed)
+  end
+
   specify "should return nil if validations fail and raise_on_save_faiure is false" do
     @m.raise_on_save_failure = false
     @m.save.should == nil
